#******************************************************************************
#
#       Copyright:      2005-2006 Paul Obermeier (obermeier@poSoft.de)
#
#                       See the file "Tcl3D_License.txt" for information on
#                       usage and redistribution of this file, and for a
#                       DISCLAIMER OF ALL WARRANTIES.
#
#       Module:         Tcl3D -> tcl3dCg
#       Filename:       tcl3dCgUtil.tcl
#
#       Author:         Paul Obermeier
#
#       Description:    Tcl module with miscellaneous Tcl3D
#                       utility procedures related to the Cg module.
#
#******************************************************************************

# Check, if a Cg related error has occured. 
# The procedure returns an empty string, if no error has occurred.
# It returns corresponding error string otherwise.

proc tcl3dCgGetError { context } {
    set err [cgGetError]

    if { $err == $::CG_NO_ERROR } {
        return ""
    }

    set ret [format "CG error: %d (%s)\n" $err [cgGetErrorString $err]]
    if { $err == $::CG_COMPILER_ERROR } {
        append ret [format "Last Listing:\n%s\n" [cgGetLastListing $context]]
    }
    return $ret
}

# Check, if a Cg related error has occured. 
# The procedure returns 0, if no error has occurred.
# It returns 1 and prints the corresponding error message
# onto standard error otherwise.

# OBSOLETE tcl3dCheckCgError 0.3.2 tcl3dCgGetError
proc tcl3dCheckCgError { context msg } {
    set err [cgGetError]

    if { $err == $::CG_NO_ERROR } {
        return 0
    }

    puts stderr [format "%s: CG error: %d (%s)\n" $msg $err [cgGetErrorString $err]]
    if { $err == $::CG_COMPILER_ERROR } {
        puts stderr [format "LAST LISTING----%s----\n" [cgGetLastListing $context]]
    }
    return 1
}

# Get a list of profile names.
# The list consists of (key,value) pairs, where key is the profile name,
# like CG_PROFILE_FP30 and value is either 1, if the corresponding profile
# is supported, or 0, if it is not available.

proc tcl3dCgGetProfileList {} {
    set profList {}
    foreach prof [lsort [info globals CG_PROFILE*]] {
        set enabled [cgGLIsProfileSupported [format "%s" $prof]]
        lappend profList $prof $enabled
    }
    return $profList
}

# OBSOLETE tcl3dGetCgProfileList 0.3.2 tcl3dCgGetProfileList
proc tcl3dGetCgProfileList {} {
    return [tcl3dCgGetProfileList]
}

# Find the first profile supported by the Cg implementation from the 
# profile names supplied in "args".
# If successful, it returns the profile name, otherwise an empty string.

proc tcl3dCgFindProfile { args } {
    foreach prof $args {
        if { [cgGLIsProfileSupported $prof] } {
            return $prof
        }
    }
    return ""
}

# OBSOLETE tcl3dFindCgProfile 0.3.2 tcl3dCgFindProfile
proc tcl3dFindCgProfile { args } {
    return [eval tcl3dCgFindProfile $args]
}

# Find a profile name by it's numerical value supplied in "profileNum".
# If successful, it returns the profile name, otherwise an empty string.
# The procedure does not check, if the profile is supported.

proc tcl3dCgFindProfileByNum { profileNum } {
    foreach prof [info globals CG_PROFILE*] {
        set cmdStr [format "::%s" $p]
        if { $profileNum == [set $cmdStr] } {
            return $p
        }
    }
    return ""
}

# OBSOLETE tcl3dFindCgProfileByNum 0.3.2 tcl3dCgFindProfileByNum
proc tcl3dFindCgProfileByNum { profileNum } {
    return [tcl3dCgFindProfileByNum $profileNum]
}

# Print the Cg program information onto standard output.

proc tcl3dCgPrintProgramInfo { progId progFile } {
    puts [format "Source file %s: Profile: %s Entry: %s" \
         $progFile \
         [cgGetProgramString $progId CG_PROGRAM_PROFILE] \
         [cgGetProgramString $progId CG_PROGRAM_ENTRY]]
}

# OBSOLETE tcl3dPrintCgProgramInfo 0.3.2 tcl3dCgPrintProgramInfo
proc tcl3dPrintCgProgramInfo { progId progFile } {
    tcl3dCgPrintProgramInfo $progId $progFile
}
