#******************************************************************************
#
#       Copyright:      2006 Paul Obermeier (obermeier@poSoft.de)
#
#                       See the file "Tcl3D_License.txt" for information on
#                       usage and redistribution of this file, and for a
#                       DISCLAIMER OF ALL WARRANTIES.
#
#       Module:         Tcl3D -> tcl3dSDL
#       Filename:       tcl3dSDLUtil.tcl
#
#       Author:         Paul Obermeier
#
#       Description:    Tcl module with miscellaneous Tcl3D
#                       utility procedures related to the SDL module.
#
#******************************************************************************

# Convert a SDL focus state bitfield into a string representation.

proc tcl3dSDLGetFocusName { state } {
    set stateStr ""
    if { $state & $::SDL_APPMOUSEFOCUS } {
        append stateStr "MouseFocus "
    } 
    if { $state & $::SDL_APPINPUTFOCUS } {
        append stateStr "InputFocus "
    } 
    if { $state & $::SDL_APPACTIVE } {
        append stateStr "Active "
    } 
    return $stateStr
}

# Convert a SDL button state bitfield into a string representation.

proc tcl3dSDLGetButtonName { state } {
    set stateStr ""
    if { $state & [expr 1 << ($::SDL_BUTTON_LEFT-1)] } {
        append stateStr "Left "
    } 
    if { $state & [expr 1 << ($::SDL_BUTTON_MIDDLE-1)] } {
        append stateStr "Middle "
    } 
    if { $state & [expr 1 << ($::SDL_BUTTON_RIGHT-1)] } {
        append stateStr "Right "
    } 
    if { $state & [expr 1 << ($::SDL_BUTTON_WHEELUP-1)] } {
        append stateStr "WheelUp "
    } 
    if { $state & [expr 1 << ($::SDL_BUTTON_WHEELDOWN-1)] } {
        append stateStr "WheelDown "
    }
    return $stateStr
}

array set ::__tcl3dSDLHatNames [list \
    $::SDL_HAT_CENTERED         "HAT_CENTERED" \
    $::SDL_HAT_UP               "HAT_UP" \
    $::SDL_HAT_RIGHT            "HAT_RIGHT" \
    $::SDL_HAT_DOWN             "HAT_DOWN" \
    $::SDL_HAT_LEFT             "HAT_LEFT" \
    $::SDL_HAT_RIGHTUP          "HAT_RIGHTUP" \
    $::SDL_HAT_RIGHTDOWN        "HAT_RIGHTDOWN" \
    $::SDL_HAT_LEFTUP           "HAT_LEFTUP" \
    $::SDL_HAT_LEFTDOWN         "HAT_LEFTDOWN" \
]

# Convert SDL hat related enumerations into a string representation.

proc tcl3dSDLGetHatName { state } {
    if { [info exists ::__tcl3dSDLHatNames($state)] } {
        return $::__tcl3dSDLHatNames($state)
    } else {
        return ""
    }
}

array set ::__tcl3dSDLEventNames [list \
    $::SDL_NOEVENT              "NOEVENT" \
    $::SDL_ACTIVEEVENT          "ACTIVEEVENT" \
    $::SDL_KEYDOWN              "KEYDOWN" \
    $::SDL_KEYUP                "KEYUP" \
    $::SDL_MOUSEMOTION          "MOUSEMOTION" \
    $::SDL_MOUSEBUTTONDOWN      "MOUSEBUTTONDOWN" \
    $::SDL_MOUSEBUTTONUP        "MOUSEBUTTONUP" \
    $::SDL_JOYAXISMOTION        "JOYAXISMOTION" \
    $::SDL_JOYBALLMOTION        "JOYBALLMOTION" \
    $::SDL_JOYHATMOTION         "JOYHATMOTION" \
    $::SDL_JOYBUTTONDOWN        "JOYBUTTONDOWN" \
    $::SDL_JOYBUTTONUP          "JOYBUTTONUP" \
    $::SDL_QUIT                 "QUIT" \
    $::SDL_SYSWMEVENT           "SYSWMEVENT" \
    $::SDL_EVENT_RESERVEDA      "EVENT_RESERVEDA" \
    $::SDL_EVENT_RESERVEDB      "EVENT_RESERVEDB" \
    $::SDL_VIDEORESIZE          "VIDEORESIZE" \
    $::SDL_VIDEOEXPOSE          "VIDEOEXPOSE" \
    $::SDL_EVENT_RESERVED2      "EVENT_RESERVED2" \
    $::SDL_EVENT_RESERVED3      "EVENT_RESERVED3" \
    $::SDL_EVENT_RESERVED4      "EVENT_RESERVED4" \
    $::SDL_EVENT_RESERVED5      "EVENT_RESERVED5" \
    $::SDL_EVENT_RESERVED6      "EVENT_RESERVED6" \
    $::SDL_EVENT_RESERVED7      "EVENT_RESERVED7" \
    $::SDL_USEREVENT            "USEREVENT" \
]

# Convert SDL event related enumerations into a string representation.

proc tcl3dSDLGetEventName { state } {
    if { [info exists ::__tcl3dSDLEventNames($state)] } {
        return $::__tcl3dSDLEventNames($state)
    } else {
        return ""
    }
}

# Utilities for the SDL CD support.

# Convert CD frames into minutes/seconds/frames.

proc tcl3dSDLFrames2MSF { frames } {
    set l [list 0 0 0]
    lset l 2 [expr $frames % $::CD_FPS]
    set frames [expr $frames / $::CD_FPS]
    lset l 1 [expr $frames%60]
    set frames [expr $frames / 60]
    lset l 0 $frames
    return $l
}

array set ::__tcl3dSDLTrackTypeNames [list \
    $::SDL_AUDIO_TRACK "AUDIO_TRACK" \
    $::SDL_DATA_TRACK  "DATA_TRACK" \
]

# Convert SDL CD track type enumerations into a string representation.

proc tcl3dSDLGetTrackTypeName { type } {
    if { [info exists ::__tcl3dSDLTrackTypeNames($type)] } {
        return $::__tcl3dSDLTrackTypeNames($type)
    } else {
        return ""
    }
}

array set ::__tcl3dSDLCdStatusNames [list \
    $::CD_TRAYEMPTY "TRAYEMPTY" \
    $::CD_STOPPED   "STOPPED" \
    $::CD_PLAYING   "PLAYING" \
    $::CD_PAUSED    "PAUSED" \
    $::CD_ERROR     "ERROR" \
]

# Convert SDL CD status enumerations into a string representation.

proc tcl3dSDLGetCdStatusName { status } {
    if { [info exists ::__tcl3dSDLCdStatusNames($status)] } {
        return $::__tcl3dSDLCdStatusNames($status)
    } else {
        return ""
    }
}
